-------------------------------------------------------------------------------
--|> EVENTS
--:: Manages all events
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitIsEnemy = UnitIsEnemy
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitIsPlayer = UnitIsPlayer
local UnitLevel = UnitLevel
local IsInInstance = IsInInstance
local bit = bit
local bit_band = bit.band
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown
local CombatLogGetCurrentEventInfo = CombatLogGetCurrentEventInfo
local UnitGUID = UnitGUID
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local strsub = strsub
local UnitIsWarModeDesired = UnitIsWarModeDesired
local wipe = wipe
local UnitExists = UnitExists
local RAID_CLASS_COLORS = RAID_CLASS_COLORS
local GetPlayerInfoByGUID = GetPlayerInfoByGUID
local GetSpecializationInfoByID = GetSpecializationInfoByID
local GetZoneText = GetZoneText
local _

-------------------------------------------------------------------------------
--|> INIT GLOBALS
-------------------------------------------------------------------------------
weizPVP.PlayerActiveCache = {}

-------------------------------------------------------------------------------
--|> FUNCTIONS
-------------------------------------------------------------------------------
--> Static Role Assignment
---------------------------------------------------------------------
local function RoleAssign(class)
    if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
        return "DAMAGER"
    end
    return nil
end

-----------------------------------------------------------------------
--> Get Spec from Spell
-----------------------------------------------------------------------
local specializationID
local function GetSpecFromSpell(SpellID)
    specializationID = weizPVP.SpecDB[SpellID]
    return specializationID or nil
end

-----------------------------------------------------------------------
--> Get Role from Spec
-----------------------------------------------------------------------

local function GetSpecRole(specid)
    if specid then
        local role
        _, _, _, _, role, _, _ = GetSpecializationInfoByID(specid)
        return role
    end
    return nil
end

-------------------------------------------------------------------------------------------
--|> DETERMINE ZONE AND INSTANCE TYPE FROM EVENTS
-------------------------------------------------------------------------------------------
local inInstance
local WarModeEnabled
--> Check for Valid (PVP) Instance
-----------------------------------------------------------------------
local function CheckValidInstance()
    --> Checking for Valid BG
    inInstance = select(2, IsInInstance()) --> Get instance info
    if inInstance == "pvp" and weizPVP.Options.Addon.EnabledInBattlegrounds == true then
        return true
    end

    if inInstance == "arena" == true and weizPVP.Options.Addon.EnabledInArena == true then
        return true
    end
    return false
end

--> Check for Valid World Zone
-----------------------------------------------------------------------
local function CheckValidWorldZone()
    --> Checking for Valid WPVP Zone
    inInstance = select(2, IsInInstance())
    WarModeEnabled = UnitIsWarModeDesired("player")
    if inInstance == "none" and WarModeEnabled == true then
        return true
    else
        return false
    end
end

--> Halt Activate
-----------------------------------------------------------------------
local function HaltActivate()
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealthEvent")
	weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
	-- weizPVP:EnableCrosshair()
end

--> Halt Deactivate
-----------------------------------------------------------------------
local function HaltDeactivate()
    if not weizPVP.Options.LagMode.Active then --> Only re-enable these events if Lagmode isn't active
        weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
        weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
        weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    end
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealthEvent")
	weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")
	-- weizPVP:DisableCrosshair()
end

--> Get PVP Zone
-----------------------------------------------------------------------
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone
function weizPVP.GetPVPZone()
    if ZoneTimerStartTime + 1 <= GetTime() then
        ZoneTimerStartTime = GetTime()
        if GetZoneText() == "" then
            C_Timer_After(0.5, weizPVP.GetPVPZone)
            return
        end
        ValidWorldZone = CheckValidWorldZone()
        ValidBG = CheckValidInstance()
        if not InCombatLockdown() then
            if ValidWorldZone or ValidBG then
                HaltDeactivate()
            else
                HaltActivate()
            end
        end
    else
        return
    end
end

--=> EVENT: Entering Battleground Instance
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringBattlegroundEvent()
    C_Timer_After(1, weizPVP.GetPVPZone)
    wipe(weizPVP.CurrentNameplates)
end

-----------------------------------------------------------------------
--=> EVENT: ZoneChangedNewAreaEvent
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
    C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
    wipe(weizPVP.CurrentNameplates)
    C_Timer_After(1, weizPVP.GetPVPZone)
end

-----------------------------------------------------------------------
--=> EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
function weizPVP:OneSecondTimer()
    --> FPS Update
    weizPVP.Display.CurrentFPS = GetFramerate()

    --> Check for LagMode
    weizPVP:LagModeCheck()

    --> Manage Timeout Lists
    weizPVP.ManageListTimeouts()
end

--> Role Check
-----------------------------------------------------------------------
local _, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, arg12, arg13
-----------------------------------------------------------------------
local function ClRoleCheck(spellID, class)
    local role = RoleAssign(class)
    if role ~= nil then
        return role
    end

    --> Check Spell ID against DB of specs/roles
    local specIdCheck = GetSpecFromSpell(spellID) or nil
    if not specIdCheck then
        return nil
    end

    --> Role
    role = GetSpecRole(specIdCheck)
    return role
end



-----------------------------------------------------------------------
--=> EVENT PULSE EVENT
-----------------------------------------------------------------------
function weizPVP:PulseEvent()
    --> FPS Update
    weizPVP.Display.CurrentFPS = GetFramerate()
    if weizPVP.Options.LagMode.HaltEnabled == true then
        if weizPVP.Display.CurrentFPS < 11 then
            HaltActivate()
        else
            HaltDeactivate()
        end
    end

    --> Manage Timeout Lists
    weizPVP.ManageListTimeouts()

    if weizPVP.Options.LagMode.Enabled then
        --> Check for LagMode
        weizPVP:LagModeCheck()
    end
end

--> STEALTH CHECK
-----------------------------------------------------------------------
local event
local stealthFound
local stealthIDs = {1784, 115834, 114018, 11327, 5215, 66, 156136}
-----------------------------------------------------------------------
local function CLStealthCheck(event, arg12, arg13, srcName)
    if event ~= "SPELL_AURA_APPLIED" or event ~= "SPELL_AURA_REFRESH" then
        return false
    end

    --> Check for match
    stealthFound = false
    for i = 1, #stealthIDs do
        if stealthIDs[i] == arg12 then
            stealthFound = true
            break
        end
    end

    --> RETURN IF NOT STEALTH
    if stealthFound == false then
        return false
    end

    --> Set Class Color
    local classColor = RAID_CLASS_COLORS[weizPVP.PlayerDB[srcName].Class] or {r = 1, g = 1, b = 1}
    --> Show Stealth Alert
    --> Enabled?
    if weizPVP.Options.StealthAlert.Enabled == true then
        weizPVP:ShowStealthAlert(arg13 .. " Detected!", dstName, classColor.r, classColor.g, classColor.b)
    end

    return true
end

-----------------------------------------------------------------------
--> Valid Combat Log Player Check
-----------------------------------------------------------------------
local function ValidLogPlayer(flags, guid, name)
    if
        (bit_band(flags, 0x00000040) ~= 0 or weizPVP.Options.DEBUG == true) and bit_band(flags, 0x00000400) ~= 0 and guid and name and
            name ~= "Unknown"
     then
        return true
    else
        return false
    end
end

-----------------------------------------------------------------------
--> CREATE NEW PLAYER ENTRY IN DB
-----------------------------------------------------------------------
local function CLogNewPlayerFound(event, srcName, srcGUID, arg12, arg13)
    if weizPVP.PlayerDB[srcName] then
        return
    end
    if srcName and srcGUID then
        weizPVP.PlayerDB[srcName] = {}
        weizPVP.PlayerDB[srcName].Name = srcName
        weizPVP.PlayerDB[srcName].Timestamp = GetTime()
        weizPVP.PlayerDB[srcName].Estimated = true
        local levelNewPlayer
        local abilityType = strsub(event, 1, 5)
        if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" then
            levelNewPlayer = weizPVP.AbilityLevelDB[arg12] or 0
        else
            levelNewPlayer = 0
        end
        weizPVP.PlayerDB[srcName].Level = levelNewPlayer
        local class
        _, class = GetPlayerInfoByGUID(srcGUID)
        weizPVP.PlayerDB[srcName].Class = class
        local newRole = ClRoleCheck(arg12, class)
        weizPVP.PlayerDB[srcName].Role = newRole or nil
    end
end

-------------------------------------------------------------------------------------------
--|> EVENTS: COMBAT LOG
-------------------------------------------------------------------------------------------
function weizPVP:CombatLogEvent()
    _, event, _, srcGUID, srcName, srcFlags, _, dstGUID, dstName, dstFlags, _, arg12, arg13, _, _, _ = CombatLogGetCurrentEventInfo()

    --> IS VALID SOURCE?
    -------------------------
    local validDst = ValidLogPlayer(dstFlags, dstGUID, dstName)
    local validSrc = ValidLogPlayer(srcFlags, srcGUID, srcName)

    if validSrc then
        --> Check if unit is not in DB yet
        --------------------------------d---------
        if not weizPVP.PlayerDB[srcName] then
            CLogNewPlayerFound(event, srcName, srcGUID, arg12, arg13)
        end

        --> Get class
        local class
        _, class = GetPlayerInfoByGUID(srcGUID)

        --> Check role
        -----------------------------------------
        local role = ClRoleCheck(arg12, class)
        if role ~= nil then
            if not weizPVP.PlayerDB[srcName].Role or weizPVP.PlayerDB[srcName].Role ~= role then
                weizPVP.PlayerDB[srcName].Role = role
            end
        end

        --> Check for dead holy priest
        -----------------------------------------
        if role == "HEALER" then
            if arg12 == 27827 or arg12 == 20711 or arg12 == 126094 then
                weizPVP:UpdatePlayerActiveCache(srcName, nil, nil, true, "HEALER", srcGUID)
                return
            end
        end

        --> Check for Stealth
        -----------------------------------------
        local stealthFound = CLStealthCheck(event, arg12, arg13, srcName)
        if stealthFound == true then
            weizPVP:UpdatePlayerActiveCache(srcName, nil, true, nil, role, srcGUID)
            return
        end

        weizPVP:UpdatePlayerActiveCache(srcName, nil, false, nil, role, srcGUID) --> Update player data: LAGMODE INACTIVE
    end

    --> VALID DST; CHECK DEAD
    -------------------------
    if validDst then
        weizPVP:CLDeath(event, dstGUID, dstName)
    end
end

-------------------------------------------------------------------------------------------
--|> EVENTS: UNITS
-------------------------------------------------------------------------------------------
--=> UNITS: MOUSEOVER
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
    if not weizPVP.Options.LagMode.Active and UnitExists("mouseover") then
        weizPVP:GetUnitData("mouseover")
    end
end

--=> UNITS: TARGET CHANGED
-- -----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
    weizPVP:GetUnitData("target")
    weizPVP:ChangeTargetIcon()
end

--=> UNITS: NAMEPLATE ADDED
-- -----------------------------------------------------------------------
function weizPVP:NameplateAdded(_, unit)
    if (UnitIsEnemy("player", unit) or weizPVP.Options.DEBUG) and UnitIsPlayer(unit) then
        local GUID = UnitGUID(unit)
        --> Update time only
        if weizPVP.PlayerActiveCache[GUID] and weizPVP.PlayerActiveCache[GUID].Estimated == false then
            weizPVP.PlayerActiveCache[GUID].Stealth = false
            weizPVP:AddPlayerDataToNearby(GUID, false)
        else
            weizPVP:GetUnitData(unit)
        end
        weizPVP.CurrentNameplates[GUID] = true
    end
end

-----------------------------------------------------------------------
--=> UNITS: NAMEPLATE REMOVED
-----------------------------------------------------------------------
function weizPVP:NameplateRemoved(_, unit)
    local GUID = UnitGUID(unit)
    if weizPVP.PlayerActiveCache[GUID] then
        weizPVP:AddPlayerDataToNearby(GUID, false)
    end
    weizPVP.CurrentNameplates[GUID] = nil
end

-----------------------------------------------------------------------
--=> EVENT: UNIT_HEALTH
-----------------------------------------------------------------------
function weizPVP:UnitHealthEvent(_, unit)
    local GUID = UnitGUID(unit) or nil
    --> Check if player is aleady on bars
    -------------------------------------
    if weizPVP.PlayersOnBars[GUID] and weizPVP.PlayerActiveCache[GUID] then
        local newhealth = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
        local dead = UnitIsDeadOrGhost(unit)

        --> If valid dead value that is different that previouosly stored; update.
        if dead ~= nil and dead ~= weizPVP.PlayerActiveCache[GUID].Dead then
            weizPVP.PlayerActiveCache[GUID].Dead = dead
        end
        if newhealth ~= nil and newhealth ~= weizPVP.PlayerActiveCache[GUID].Health then
            weizPVP.PlayerActiveCache[GUID].Health = newhealth
        end
    end
end

-------------------------------------------------------------------------------------------
--|> EVENTS: COMBAT
-------------------------------------------------------------------------------------------
--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
    weizPVP:SetStatusBarMessage("|cFFF4564D+++ IN COMBAT +++|r", false, 0)
end

-----------------------------------------------------------------------
--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
    weizPVP:SetStatusBarMessage("|cFF53F493--- OUT OF COMBAT ---|r", true, 3)
    weizPVP:ManageListTimeouts()
end

-----------------------------------------------------------------------
--=> EVENT: PLAYER LEVELED UP
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
    weizPVP.Player.Level = UnitLevel("player") --> Update player level
end

--> COMBAT LOG DEATH CHECK
-----------------------------------------------------------------------
function weizPVP:CLDeath(fEvent, fGUID, fName)
    if weizPVP.PlayerActiveCache[fGUID] then
        if fEvent == "UNIT_DIED" or fEvent == "UNIT_DESTROYED" or fEvent == "UNIT_DISSIPATES" or fEvent == "PARTY_KILL" then
            weizPVP.PlayerActiveCache[fGUID].Dead = true
            weizPVP:UpdatePlayerActiveCache(fName, nil, nil, true, nil, fGUID)
            return true
        end
    end
    return false --> return if both src and dst are known
end

-----------------------------------------------------------------------
--> UPDATE DISPLAY DATA
-----------------------------------------------------------------------
function weizPVP:UpdateDisplayData()
    weizPVP.Display.Width = GetScreenWidth()
    weizPVP.Display.Height = GetScreenHeight()
    weizPVP.Display.xFactor = (768 / weizPVP.Display.Width) * (weizPVP.Display.Width / weizPVP.Display.Height)
    weizPVP.Display.yFactor = 768 / weizPVP.Display.Height
    weizPVP.Display.UIScale = UIParent:GetScale()
    weizPVP.Display.EffectiveScale = UIParent:GetEffectiveScale()
end

-----------------------------------------------------------------------
--> ENABLE EVENTS
-----------------------------------------------------------------------
function weizPVP:EnableEvents()
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealthEvent")

    --> ZONE CHANGED
    weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", "PlayerEnteringBattlegroundEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")

    --> COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:RegisterEvent("PLAYER_LOGIN", "UpdateDisplayData")

    --> Crosshairs
    weizPVP:EnableCrosshair()
end

-----------------------------------------------------------------------
--> DISABLE EVENTS
-----------------------------------------------------------------------
function weizPVP:DisableEvents()
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealthEvent")

    --> COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")

    --> PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")

    --> DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", "UpdateDisplayData")
    weizPVP:UnregisterEvent("PLAYER_LOGIN", "UpdateDisplayData")

    -->Crosshairs
    weizPVP:DisableCrosshair()
end

-------------------------------------------------------------------------------------------
--|>   LAGMODE
-------------------------------------------------------------------------------------------
--> ACTIVATE
local function LagModeActivate()
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP.CurrentNameplates = {}
    weizPVP.Options.LagMode.Active = true
end

--> DEACTIVATE
local function LagModeDeactivate()
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP.Options.LagMode.Active = false
end

-------------------------------------------------------------------------------------------
--|>   LAGMODE CHECK
-------------------------------------------------------------------------------------------
local LagModeCounter = 0

function weizPVP:LagModeCheck()
    if weizPVP.Options.LagMode.Enabled == true then
        if LagModeCounter >= 4 then
            LagModeCounter = 0
            if weizPVP.Display.CurrentFPS < weizPVP.Options.LagMode.FPSThreshold then
                LagModeActivate()
            elseif weizPVP.Display.CurrentFPS > weizPVP.Options.LagMode.FPSThreshold then
                if weizPVP.Options.LagMode.Active == true then
                    LagModeDeactivate()
                end
            end
        else
            LagModeCounter = LagModeCounter + 1
        end
    elseif weizPVP.Options.LagMode.Active == true then
        LagModeDeactivate()
    end
end
