-------------------------------------------------------------------------------
--|> FRAMES
--:: Creates and updates the addon frames and widgets
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local InCombatLockdown = InCombatLockdown
local GetTime = GetTime
local IsControlKeyDown = IsControlKeyDown
local UIParent = UIParent
local CreateFrame = CreateFrame
local UIFrameFlash = UIFrameFlash
local tostring = tostring
local UIFrameFadeOut = UIFrameFadeOut
local UIFrameFadeIn = UIFrameFadeIn

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

-------------------------------------------------------------------------------
--|> LOCAL FUNCTIONS
-------------------------------------------------------------------------------
--=> Adjust Scroll Offset
-------------------------------------------------------------------------------
local offsetMax
local offset
function weizPVP.AdustScrollOffset(self, value)
    if not InCombatLockdown() then
        local vertScroll = weizPVP.ScrollFrame:GetVerticalScroll()
        local RowHeight = weizPVP.Options.Bars.RowHeight + weizPVP.Options.Bars.VerticalSpacing
        offsetMax = weizPVP.ListFrame:GetHeight() - weizPVP.ScrollFrame:GetHeight()
        offset = 0
        if value > 0 then
            offset = vertScroll - (RowHeight)
        else
            offset = vertScroll + (RowHeight)
        end
        if offset < 0 then
            weizPVP.ScrollFrame:SetVerticalScroll(0)
        elseif offset > offsetMax then
            weizPVP.ScrollFrame:SetVerticalScroll(offsetMax)
        else
            weizPVP.ScrollFrame:SetVerticalScroll(offset)
        end
    end
end

--=> AlignFrameToPixels
-----------------------------------------------------------------------
local x, y
local fScale
function weizPVP:AlignFrameToPixels(frame)
    if not InCombatLockdown() then
        local effectiveScale = frame:GetEffectiveScale()
        x, y = frame:GetCenter()
        if x and y then
            x, y = x * effectiveScale, y * effectiveScale
            fScale = frame:GetScale()
            x, y = x / fScale, y / fScale
            frame:SetPoint("CENTER", nil, "BOTTOMLEFT", weizPVP.ScaleCoords(x, y))
        end
    end
end

-------------------------------------------------------------------------------
--|> CONFIGURE HEADER
-------------------------------------------------------------------------------

function weizPVP:CreateCoreFrames()
    --> weizPVP.CoreFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.CoreFrame = CreateFrame("Frame", "weizPVP.CoreFrame", UIParent)
    weizPVP.CoreFrame:SetClampedToScreen(true)
    weizPVP.CoreFrame:SetClipsChildren(true)
    weizPVP.CoreFrame:SetFrameStrata("MEDIUM")
    weizPVP.CoreFrame:SetPoint("CENTER", UIParent)
    weizPVP.CoreFrame:SetSize(251, 247)
    weizPVP.CoreFrame:SetMovable(true)
    weizPVP.CoreFrame:EnableMouse(true)
    weizPVP.CoreFrame:SetDontSavePosition(true)
    weizPVP.CoreFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", weizPVP.Options.Frames.BackgroundTexture),
            tile = false
        }
    )
    weizPVP.CoreFrame:SetBackdropColor(unpack(weizPVP.Options.Frames.BackgroundColor))
    weizPVP.CoreFrame:SetResizable(true)
    weizPVP.CoreFrame:SetMinResize(180, weizPVP.Options.Bars.RowHeight)
    weizPVP.CoreFrame:SetMaxResize(500, weizPVP.Options.Frames.List.Height)
    weizPVP.CoreFrame:Show()

    --> weizPVP.ScrollFrame: SETTINGS
    --: The frame that is a child of weizPVP.CoreFrame that will be scolling with content
    -----------------------------------------------------------------------
    weizPVP.ScrollFrame = CreateFrame("ScrollFrame", "weizPVP.ScrollFrame", weizPVP.CoreFrame)
    weizPVP.ScrollFrame:EnableMouseWheel(true)
    weizPVP.ScrollFrame:SetClipsChildren(true)
    weizPVP.ScrollFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "TOPLEFT", weizPVP.Options.Frames.BorderSize, 0)
    weizPVP.ScrollFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT", (-1 * weizPVP.Options.Frames.BorderSize), 0)
    --> MOUSEWHEEL
    weizPVP.ScrollFrame:SetScript(
        "OnMouseWheel",
        function(self, value)
            weizPVP.AdustScrollOffset(self, value)
        end
    )
    --> weizPVP.ListFrame: SETTINGS
    --: The frame that is a child of weizPVP.CoreFrame that will be scolling with content
    -----------------------------------------------------------------------
    weizPVP.ListFrame = CreateFrame("Frame", "weizPVP.ListFrame", weizPVP.ScrollFrame)
    weizPVP.ListFrame:SetPoint("CENTER", weizPVP.ScrollFrame, "CENTER", 0, 1)
    weizPVP.ListFrame:SetPoint("TOP", weizPVP.CoreFrame, "BOTTOM", 0, 1)
    weizPVP.ListFrame:SetHeight(weizPVP.Options.Frames.List.Height)
    weizPVP.ListFrame:SetWidth(weizPVP.ScrollFrame:GetWidth() - weizPVP.Options.Frames.BorderSize)
    weizPVP.ScrollFrame:SetScrollChild(weizPVP.ListFrame)

    --> weizPVP.HeaderFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame = CreateFrame("Frame", "weizPVP.HeaderFrame", UIParent)
    weizPVP.HeaderFrame:SetClampedToScreen(true)
    weizPVP.HeaderFrame:SetFrameStrata("MEDIUM")
    weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
    weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
    weizPVP.HeaderFrame:SetHeight(weizPVP.Options.Frames.Header.Height)
    weizPVP.HeaderFrame:SetDontSavePosition(true)
    weizPVP.HeaderFrame:SetFrameLevel(0)
    weizPVP.HeaderFrame:RegisterForDrag("LeftButton")
    weizPVP.HeaderFrame:EnableMouse(true)
    weizPVP.HeaderFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPVP: Border"),
            edgeSize = 1,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    weizPVP.HeaderFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.8)
    weizPVP.HeaderFrame:SetBackdropBorderColor(0, 0, 0, 1)

    --> Header Frame Highlight
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame.Highlight = CreateFrame("Frame", "weizPVP:HeaderFrame.Highlight", weizPVP.HeaderFrame)
    weizPVP.HeaderFrame.Highlight:SetPoint("TOPLEFT", weizPVP.HeaderFrame, "TOPLEFT", 1, -1)
    weizPVP.HeaderFrame.Highlight:SetPoint("BOTTOMRIGHT", weizPVP.HeaderFrame, "BOTTOMRIGHT", -1, 1)
    weizPVP.HeaderFrame.Highlight:SetFrameStrata("MEDIUM")
    weizPVP.HeaderFrame.Highlight:SetFrameLevel(1)
    weizPVP.HeaderFrame.Highlight:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: #FFFFFF"),
            tile = false
        }
    )
    weizPVP.HeaderFrame.Highlight:SetBackdropColor(0.8, 0.2, 0.2, 0.6)
    weizPVP.HeaderFrame.Highlight:EnableMouse(false)
    weizPVP.HeaderFrame.Highlight:Show()
    weizPVP.HeaderFrame.Highlight:SetAlpha(0)

    --> weizPVP.HeaderFrame.Text: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame.Text = weizPVP.HeaderFrame:CreateFontString(nil, "MEDIUM", nil)
    weizPVP.HeaderFrame.Text:SetPoint("LEFT", weizPVP.HeaderFrame, "LEFT", 8, 0)
    weizPVP.HeaderFrame.Text:SetFont(
        SM:Fetch("font", weizPVP.Options.Frames.Header.Font),
        weizPVP.Options.Frames.Header.FontSize,
        weizPVP.Options.Frames.Header.FontOutline
    )
    weizPVP.HeaderFrame.Text:SetJustifyH("LEFT")
    weizPVP.HeaderFrame.Text:SetJustifyV("MIDDLE")
    weizPVP.HeaderFrame.Text:SetDrawLayer("OVERLAY")
    weizPVP.HeaderFrame.Text:SetTextColor(1, 1, 1, 1)
    weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFF26FF540|r")
    weizPVP.HeaderFrame.Text:SetScale(1)

    --> weizPVP.HeaderFrame: SCRIPTS
    -----------------------------------------------------------------------
    --> ON DRAG START
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnDragStart",
        function()
            if weizPVP.Options.Window.Locked == false and weizPVP.Options.Window.Pinned == false then
                weizPVP.HeaderFrame:SetAlpha(0.8)
                weizPVP.CoreFrame:SetAlpha(0.8)
            end
        end
    )
    --> ON DRAG START
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnDragStop",
        function()
            weizPVP.HeaderFrame:SetAlpha(1)
            weizPVP.CoreFrame:SetAlpha(1)
        end
    )
    --> weizPVP.HeaderFrame: SCRIPTS
    -----------------------------------------------------------------------
    --> ON MOUSE DOWN
    local lastTry = 0
    local timeBetweenChatMessage = 5
    -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnMouseDown",
        function(_, button)
            if button == "LeftButton" then
                if weizPVP.Options.Window.Locked == true then --> WINDOW LOCKED; WILL NOT MOVE.
                    weizPVP:SetStatusBarMessage("Window is |cffff3838LOCKED|r", true, 1)
                    UIFrameFlash(weizPVP.HeaderFrame.Highlight, 0.1, 0.6, 0.7)
                    if lastTry + timeBetweenChatMessage < GetTime() then
                        weizPVP:PrintAddonMessage("Window is |cffff3838LOCKED|r. Use Ctrl + Right click on the title bar to unlock it.")
                        lastTry = GetTime()
                    end
                elseif weizPVP.Options.Window.Pinned == true then --> WINDOW PINNED; CLICKING THE TITLE BAR WILL NOT CHANGE THAT,.
                    weizPVP:SetStatusBarMessage("Window is |cFFF4564DPINNED|r", true, 1)
                    UIFrameFlash(weizPVP.HeaderFrame.Highlight, 0.1, 0.6, 0.7)
                    if lastTry + timeBetweenChatMessage < GetTime() then
                        weizPVP:PrintAddonMessage("Window is |cffff3838PINNED|r. Right click the title bar to unpin.")
                        lastTry = GetTime()
                    end
                else
                    weizPVP.CoreFrame:StartMoving()
                    weizPVP.CoreFrame:SetClampRectInsets(
                        0,
                        0,
                        weizPVP.Options.Frames.Header.Height + 1,
                        -1 * (weizPVP.Options.Frames.Footer.Height)
                    )
                    weizPVP.CoreFrame.isMoving = true
                end
            elseif button == "RightButton" then
                if IsControlKeyDown() then
                    weizPVP:SetWindowLocked(not weizPVP.Options.Window.Locked)
                elseif weizPVP.Options.Window.Locked == true then
                    weizPVP:SetStatusBarMessage("Window is |cffff3838LOCKED|r", true, 1)
                    if lastTry + timeBetweenChatMessage < GetTime() then
                        weizPVP:PrintAddonMessage("Window is |cffff3838LOCKED|r. Use Ctrl + Right click on the title bar to unlock it.")
                        lastTry = GetTime()
                    end
                elseif weizPVP.Options.Window.Pinned == true then
                    weizPVP:SetWindowPin(false)
                else
                    weizPVP:SetWindowPin(true)
                end
            end
        end
    )
    --> ON MOUSE UP
    -- -------------------------------------------
    weizPVP.HeaderFrame:SetScript(
        "OnMouseUp",
        function(_, button)
            weizPVP.CoreFrame:StopMovingOrSizing()
            weizPVP.CoreFrame.isMoving = false
            weizPVP:SaveCoreFramePosition()
        end
    )

    --> weizPVP.OuterIcons
    --: Frame that contains icons outside of the core frame
    -----------------------------------------------------------------------
    weizPVP.OuterIconsFrame = CreateFrame("Frame", "weizPVP.OuterIconsFrame", weizPVP.HeaderFrame)
    weizPVP.OuterIconsFrame:SetPoint("TOPLEFT", weizPVP.ScrollFrame, "TOPLEFT", -24, 0)
    weizPVP.OuterIconsFrame:SetPoint("BOTTOMRIGHT", weizPVP.ScrollFrame, "BOTTOMRIGHT", 0, 0)
    weizPVP.OuterIconsFrame:SetClipsChildren(true)

    --> weizPVP. Create Header Icons
    -----------------------------------------------------------------------
    weizPVP:CreateHeaderIcons()

    --> weizPVP.FooterFrame: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.FooterFrame = CreateFrame("Frame", "weizPVP.FooterFrame", weizPVP.HeaderFrame)
    weizPVP.FooterFrame:SetClampedToScreen(true)
    weizPVP.FooterFrame:SetFrameStrata("MEDIUM")
    weizPVP.FooterFrame:SetFrameLevel(1)
    weizPVP.FooterFrame:SetPoint("TOPLEFT", weizPVP.CoreFrame, "BOTTOMLEFT")
    weizPVP.FooterFrame:SetPoint("TOPRIGHT", weizPVP.CoreFrame, "BOTTOMRIGHT")
    weizPVP.FooterFrame:EnableMouse(false)
    weizPVP.FooterFrame:SetHeight(weizPVP.Options.Frames.Footer.Height)
    weizPVP.FooterFrame:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPVP: Border"),
            edgeSize = 1,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    weizPVP.FooterFrame:SetBackdropColor(0.01, 0.01, 0.01, 0.7)
    weizPVP.FooterFrame:SetBackdropBorderColor(0, 0, 0, 1)
    weizPVP.FooterFrame:SetScale(1)

    --> weizPVP.FooterFrame.Text: SETTINGS
    -----------------------------------------------------------------------
    weizPVP.FooterFrame.Text = weizPVP.FooterFrame:CreateFontString(nil, "OVERLAY", nil)
    weizPVP.FooterFrame.Text:SetPoint("BOTTOMLEFT", weizPVP.FooterFrame, "BOTTOMLEFT", 2, 1)
    weizPVP.FooterFrame.Text:SetFont(SM:Fetch("font", weizPVP.Options.Frames.Footer.Font), 16, "OUTLINE")
    weizPVP.FooterFrame.Text:SetJustifyH("LEFT")
    weizPVP.FooterFrame.Text:SetJustifyV("MIDDLE")
    weizPVP.FooterFrame.Text:SetText("")

    weizPVP.FooterFrame.Flash = CreateFrame("Frame", "weizPVP.FooterFrame", weizPVP.FooterFrame)
    weizPVP.FooterFrame.Flash:SetFrameStrata("MEDIUM")
    weizPVP.FooterFrame.Flash:SetFrameLevel(2)
    weizPVP.FooterFrame.Flash:SetAllPoints()

    --> Header Frame Highlight
    -----------------------------------------------------------------------
    weizPVP.HeaderFrame.Highlight = CreateFrame("Frame", "weizPVP:HeaderFrame.Highlight", weizPVP.HeaderFrame)
    weizPVP.HeaderFrame.Highlight:SetPoint("TOPRIGHT", weizPVP.HeaderFrame, "TOPRIGHT", -1, -1)
    weizPVP.HeaderFrame.Highlight:SetPoint("BOTTOMLEFT", weizPVP.HeaderFrame, "BOTTOMLEFT", 1, 1)
    weizPVP.HeaderFrame.Highlight:SetFrameStrata("MEDIUM")
    weizPVP.HeaderFrame.Highlight:SetFrameLevel(2)
    weizPVP.HeaderFrame.Highlight:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: #FFFFFF"),
            tile = false
        }
    )
    weizPVP.HeaderFrame.Highlight:SetBackdropColor(1, 0.2, 0.2, 0.6)
    weizPVP.HeaderFrame.Highlight:EnableMouse(false)
    weizPVP.HeaderFrame.Highlight:Show()
    weizPVP.HeaderFrame.Highlight:SetAlpha(0)

    weizPVP:CreateResizeGrip()

    if weizPVP.Options.Frames.Scale then
        weizPVP.CoreFrame:SetScale(weizPVP.Options.Frames.Scale)
        weizPVP.HeaderFrame:SetScale(weizPVP.Options.Frames.Scale)
    end
end

-----------------------------------------------------------------------
--> Save Core Position
-----------------------------------------------------------------------
function weizPVP:SaveCoreFramePosition()
    weizPVP.Options.Frames.X = weizPVP.CoreFrame:GetLeft()
    weizPVP.Options.Frames.Y = weizPVP.CoreFrame:GetTop()
    weizPVP.Options.Frames.Width = weizPVP.FooterFrame:GetWidth()
    weizPVP.Options.Frames.Height = weizPVP.CoreFrame:GetHeight()
end

-----------------------------------------------------------------------
--> Set Core Position
-----------------------------------------------------------------------
function weizPVP:SetCoreFramePosition(x, y, width, height)
    weizPVP.CoreFrame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x, y)
    weizPVP.HeaderFrame:ClearAllPoints()
    weizPVP.HeaderFrame:SetPoint("BOTTOMLEFT", weizPVP.CoreFrame, "TOPLEFT")
    weizPVP.HeaderFrame:SetPoint("BOTTOMRIGHT", weizPVP.CoreFrame, "TOPRIGHT")
    weizPVP.CoreFrame:SetWidth(width)
    weizPVP.CoreFrame:SetHeight(height)
    weizPVP:SaveCoreFramePosition()
end

-----------------------------------------------------------------------
--> Window: Set Locked
-----------------------------------------------------------------------
function weizPVP:SetWindowLocked(value)
    if value == nil then
        return
    end
    weizPVP.Options.Window.Locked = value

    if value == true then
        weizPVP.CoreFrame:SetMovable(false)
        weizPVP.HeaderFrame:SetMovable(false)
        weizPVP.Options.Window.Locked = true
        weizPVP:SetStatusBarMessage("Window is now |cffff3838LOCKED|r", true, 1)
    else
        weizPVP.CoreFrame:SetMovable(true)
        weizPVP.HeaderFrame:SetMovable(true)
        weizPVP.Options.Window.Locked = false
        weizPVP.Options.Window.Pinned = false
        weizPVP:SetStatusBarMessage("Window is now |cff37ff37UNLOCKED|r", true, 1)
    end
    return
end

--> Window: Set Pin
-----------------------------------------------------------------------
function weizPVP:SetWindowPin(value)
    if value == nil then
        return
    end
    if value == true then
        weizPVP.Options.Window.Pinned = true
        weizPVP.HeaderFrame:SetMovable(false)
        weizPVP:SetStatusBarMessage("Window is now |cFFF4564DPINNED|r", true, 1)
    elseif value == false then
        weizPVP.Options.Window.Pinned = false
        weizPVP.HeaderFrame:SetMovable(true)
        weizPVP:SetStatusBarMessage("Window is now |cff42dcf4UNPINNED|r", true, 1)
    end
    return
end

-------------------------------------------------------------------------------
--> Splash Frame Generate
-------------------------------------------------------------------------------
function weizPVP:CreateSplashScreen()
    --> CREATE SPLASH WINDOW
    local f = CreateFrame("Frame", "weizPVPSplashScreen", UIParent)
    f:SetFrameStrata("MEDIUM")
    f:SetPoint("CENTER", 0, 60)
    f:SetSize(500, weizPVP.CoreFrame:GetHeight() + 26 + 18)
    f:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: Gradient"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPVP: Border"),
            edgeSize = 2,
            insets = {left = 0, right = 0, top = 0, bottom = 0}
        }
    )
    f:SetBackdropColor(0.08, 0.08, 0.08, 0.9)
    f:SetBackdropBorderColor(0, 0, 0, 1)
    f:SetAlpha(0)
    f:SetClampedToScreen(true)
    f:SetMovable(true)
    f:EnableMouse(true)
    f:RegisterForDrag("LeftButton")
    f:SetDontSavePosition(true)
    f:SetScript(
        "OnDragStart",
        function()
            f:StartMoving()
            f:SetAlpha(0.8)
        end
    )
    f:SetScript(
        "OnDragStop",
        function()
            f:StopMovingOrSizing()
            f:SetAlpha(1)
        end
    )

    f:Hide()

    --> VERSION
    f.version = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    f.version:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 6)
    f.version:SetJustifyH("RIGHT")
    f.version:SetFont(SM:Fetch("font", "Accidental Presidency"), 16, "NONE")
    f.version:SetText("weizPVP - " .. "|cffffa012" .. tostring(weizPVP.Addon.VERSION .. "|r  "))

    --> TITLE
    f.title = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    f.title:SetFont(SM:Fetch("font", "Roboto Condensed BoldItalic"), 28, "OUTLINE")
    f.title:SetTextColor(1, 1, 1)
    f.title:SetJustifyH("CENTER")
    f.title:SetPoint("TOP", 0, -2)
    f.title:SetText("|cffffa012Thanks for using|r |cffffffffweiz|r|cffffa012PVP!|r \n")

    --> CHANGELOG TITLE
    f.changeTitle = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    f.changeTitle:SetPoint("TOP", f.title, "BOTTOM", 0, 0)
    f.changeTitle:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 20, "OUTLINE")
    f.changeTitle:SetWidth(480)
    f.changeTitle:SetJustifyH("LEFT")
    f.changeTitle:SetText("|cff42dcf4Recent Changes|r:")

    --> CHANGELOG
    f.changes = f:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
    f.changes:SetPoint("TOP", f.changeTitle, "BOTTOM", 0, -4)
    f.changes:SetFont(SM:Fetch("font", "PT Sans"), 14, "NONE")
    f.changes:SetWidth(480)
    f.changes:SetJustifyH("LEFT")
    f.changes:SetText(
        "|cffF61983 > ALL OPTIONS AND PLAYER DATA HAVE BEEN RESET < |r\n" ..
            "  |cffaaaaaa  * Options and Player Data is now properly stored separately.|r\n" ..
                "  |cffaaaaaa  * No more data resets required for future version.|r\n" ..
                    "  |cffaaaaaa  * Any future changes will involve migration code.|r\n \n" ..
                        "+ Many optimizations and bug fixes just about everywhere.\n" ..
                            "+ Health updating optimized greatly after noticing a new API bug.\n" ..
                                "+ Crosshair optimizations, fixes, improvments, and additional options..\n" ..
                                    "+ Better notifications of window locking and pinning.\n" .. "+ More sound files added for alerts."
    )

    --> OK BUTTON
    f.okayButton = CreateFrame("Button", "weizPVP: Splash OK Button", f)
    f.okayButton:SetSize(96, 28)
    f.okayButton:SetPoint("BOTTOM", 0, 10)
    f.okayButton:SetNormalTexture(SM:Fetch("background", "weizPVP: Gradient"))
    f.okayButton:SetHighlightTexture(SM:Fetch("background", "weizPVP: #FFFFFF"))
    f.okayButton:GetHighlightTexture():SetVertexColor(1, 1, 1, 0.1)
    f.okayButton:GetNormalTexture():SetVertexColor(0.4, 0.4, 0.4, 1)
    f.okayButton:SetBackdrop(
        {
            bgFile = SM:Fetch("background", "weizPVP: Background"),
            tile = false,
            edgeFile = SM:Fetch("border", "weizPVP: Border"),
            edgeSize = 2,
            insets = {left = -1, right = -1, top = -1, bottom = -1}
        }
    )
    f.okayButton:SetBackdropBorderColor(0, 0, 0, 1)
    f.okayButton:SetBackdropColor(0, 0, 0, 1)
    f.okayButton:SetScript(
        "OnClick",
        function()
            --> Keep okay button values the same while fading out
            f.okayButton:GetNormalTexture():SetVertexColor(0.8, 0.8, 0.8, 1)
            f.okayButton.text:SetTextColor(0.8, 0.8, 0.8, 1)
            --> CLEAN ALL SETTINGS
            f:SetMovable(false)
            f:EnableMouse(false)
            --> LOAD CHARACTER OPTIONS
            weizPVP.charDB:ResetDB()
			-- wipe(weizPVP.Options)
			-- weizPVP:charDB.SetDefaults()
			weizPVP.globalDB:ResetDB()
            -- wipe(weizPVP.PlayerDB)
            weizPVP.charDB = LibStub("AceDB-3.0"):New("weizPVPOptions", weizPVP.defaults, true)
            weizPVP.Options = weizPVP.charDB.profile.Options
            --> LOAD GLOBAL PLAYER DB
            weizPVP.globalDB = LibStub("AceDB-3.0"):New("weizPVPDatabase", {}, false)
            weizPVP.PlayerDB = weizPVP.globalDB.global
            --> Fade out
            UIFrameFadeOut(f.okayButton, 0.3, 1, 0)
            UIFrameFadeOut(f, 0.3, 1, 0)
            weizPVP.Options.Addon.VersionSplashSeen = true
        end
    )
    --> OK TEXT
    f.okayButton.text = f.okayButton:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
    f.okayButton.text:SetPoint("CENTER")
    f.okayButton.text:SetJustifyH("CENTER")
    f.okayButton.text:SetTextColor(0.9, 0.9, 0.9, 1)
    f.okayButton.text:SetText("OK")
    f.okayButton.text:SetFont(SM:Fetch("font", "Roboto Condensed Bold"), 20, "NONE")

    f.okayButton:SetScript(
        "OnEnter",
        function()
            UIFrameFadeIn(f.okayButton:GetNormalTexture():SetVertexColor(0.6, 0.6, 0.6, 1), 0.3, 1, 0)
        end
    )
    f.okayButton:SetScript(
        "OnLeave",
        function()
            f.okayButton:GetNormalTexture():SetVertexColor(0.4, 0.4, 0.4, 1)
        end
    )

    weizPVP.splash = f
    weizPVP:SetCoreFramePosition(1248, 724, weizPVP.FooterFrame:GetWidth(), weizPVP.CoreFrame:GetHeight())
    f:ClearAllPoints()
    f:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", 1248 - 20, 724 + 26)
end

function weizPVP:CheckSplash()
    --> Check to see if new version popup has been seen
    if weizPVP.Options.Addon.VersionSplashSeen == false or weizPVP.Options.DEBUG == true then
        UIFrameFadeIn(weizPVP.splash, 1, 0, 1)
    else
        weizPVP.splash = nil
    end
end
