-------------------------------------------------------------------------------
--|> Utils
-- Utilities for common functions.
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local string_format = string.format
local math_floor = math.floor

-------------------------------------------------------------------------------
--|> COLOR FUNCTIONS
-------------------------------------------------------------------------------

--> RGB TO HEX
-----------------------------------------------------------
function weizPVP:RGBToHex(r, g, b)
    r = r <= 255 and r >= 0 and r or 0
    g = g <= 255 and g >= 0 and g or 0
    b = b <= 255 and b >= 0 and b or 0
    return string_format("%02x%02x%02x", r, g, b)
end

--> Adjust RGB Tint
-----------------------------------------------------------
local r, g, b
local valueAbs
local h, s, v
local cv, cr, cg, cb
function weizPVP:AdjustRGBTint(rgb, value)
    r, g, b = unpack(rgb)
    valueAbs = math.abs(value)
    h, s, v = weizPVP.RGBToHSV(r, g, b)
    cv = v
    if value > 0 then
        cv = math.max((cv + value), 0.8)
    else
        cv = math.min((cv + value), 1 - valueAbs)
    end

    cr, cg, cb = weizPVP.HSVToRGB(h, s, cv)

    return cr, cg, cb
end

--> RGB TO HSV
-----------------------------------------------------------
local mincolor, maxcolor
local ch, cs, cv
local delta
function weizPVP.RGBToHSV(r, g, b)
    mincolor, maxcolor = math.min(r, g, b), math.max(r, g, b)
    ch, cs, cv = 0, 0, maxcolor
    if maxcolor > 0 then -- technically ch is undefined if cs is zero
        delta = maxcolor - mincolor
        cs = delta / maxcolor
        if delta > 0 then -- don't allow divide by zero
            if r == maxcolor then
                ch = (g - b) / delta -- between yellow and magenta
            elseif g == maxcolor then
                ch = 2 + ((b - r) / delta) -- between cyan and yellow
            else
                ch = 4 + ((r - g) / delta) -- between magenta and cyan
            end
        end
        if ch < 0 then
            ch = ch + 6
        end -- correct for negative values
        ch = ch / 6 -- and finally adjust range 0 to 1.0
    end
    return ch, cs, cv
end

--> HSV TO RGB
-----------------------------------------------------------
local sextant
local fract
local p, q, t
function weizPVP.HSVToRGB(ch, cs, cv)
    if not ch or not cs or not cv then
        return 1, 0, 0
    end
    if ch == 1 then
        ch = 0
    end
    r, g, b = cv, cv, cv
    if cs > 0 then -- if cs is zero then grey is returned
        h = ch * 6
        sextant = math.floor(h) -- figure out which sextant of the color wheel
        fract = h - sextant -- fractional offset into the sextant
        p, q, t = cv * (1 - cs), cv * (1 - (cs * fract)), cv * (1 - (cs * (1 - fract)))
        if sextant == 0 then
            r, g, b = cv, t, p
        elseif sextant == 1 then
            r, g, b = q, cv, p
        elseif sextant == 2 then
            r, g, b = p, cv, t
        elseif sextant == 3 then
            r, g, b = p, q, cv
        elseif sextant == 4 then
            r, g, b = t, p, cv
        else
            r, g, b = cv, p, q
        end
    end
    return r, g, b
end

-------------------------------------------------------------------------------
--|> MATH FUNCTIONS
-------------------------------------------------------------------------------

--=> Round
-----------------------------------------------------------
local mult
function weizPVP:Round(number, decimal)
    mult = 10 ^ (decimal or 0)
    return math_floor(number * mult + 0.5) / mult
end

-------------------------------------------------------------------------------
--|> CHAT FUNCTIONS
-------------------------------------------------------------------------------

--=> Send Message to Chat
-----------------------------------------------------------
local msgPrefix = "|cffeaeaeaweiz|r|cffffa012PVP|r : "
function weizPVP:PrintAddonMessage(msg)
    --> Print message with default addon prefix
    (SELECTED_CHAT_FRAME or DEFAULT_CHAT_FRAME):AddMessage(msgPrefix .. msg)
end
